package com.mongo.reactivie.controller;

import com.mongo.reactivie.model.JobPost;
import com.mongo.reactivie.model.JobPostCompanyDetails;
import com.mongo.reactivie.service.JobService;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
public class JobPostController {

    @Autowired
    private JobService jobService;

    @GetMapping("/posts")
    @ResponseStatus(HttpStatus.OK)
    public Flux<JobPost> getAllPosts() {
        return jobService.findAll();
    }

    @GetMapping("/posts/{id}")
    @ResponseStatus(HttpStatus.OK)
    public Mono<JobPost> getPostById(@PathVariable("id") String id) {
        return jobService.findById(id);
    }

    @PostMapping("/posts")
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<JobPost> createPost(@RequestBody JobPost jobPost) {
        return jobService.save(jobPost);
    }

    @PostMapping("/posts/saveall")
    @ResponseStatus(HttpStatus.CREATED)
    public Flux<JobPost> saveAll(@RequestBody List<JobPost> jobPosts) {
        return jobService.saveAll(jobPosts);
    }

    @PutMapping("/posts/{id}")
    @ResponseStatus(HttpStatus.OK)
    public Mono<JobPost> updatePost(@PathVariable("id") String id, @RequestBody JobPost jobPost) {
        return jobService.update(id, jobPost);
    }

    @DeleteMapping("/posts/{id}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public Mono<Void> deletePost(@PathVariable("id") String id) {
        return jobService.deleteById(id);
    }

    @GetMapping("/publishMessage/{id}")
    @ResponseStatus(HttpStatus.OK)
    public Mono<String> publishMessage(@PathVariable("id") String id) {
        return jobService.publishMessage(id);
    }

    @GetMapping("/posts/getJobPostCompanyDetails")
    @ResponseStatus(HttpStatus.OK)
    public Mono<List<JobPostCompanyDetails>> getJobPostCompanyDetails() {
        return Mono.just(jobService.getJobPostCompanyDetails());
    }

}
