package com.mongo.reactivie.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongo.reactivie.model.JobPost;
import com.mongo.reactivie.model.JobPostCompanyDetails;
import com.mongo.reactivie.repository.JobPostDAOImpl;
import com.mongo.reactivie.repository.JobPostRepository;
import java.util.List;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class JobService {

    @Autowired
    private JobPostRepository jobPostRepository;

    @Autowired
    private MessagePublishService messagePublishService;

    @Autowired
    private JobPostDAOImpl jobPostDAOImpl;

    public Flux<JobPost> findAll() {
        return jobPostRepository.findAll();
    }

    public Mono<JobPost> findById(String id) {
        return jobPostRepository.findById(id);
    }

    public Mono<JobPost> save(JobPost jobPost) {
        return jobPostRepository.save(jobPost);
    }

    public Flux<JobPost> saveAll(List<JobPost> jobPosts) {
        return jobPostRepository.saveAll(jobPosts);
    }

    public Mono<JobPost> update(String id, JobPost jobPost) {
        return jobPostRepository.findById(id).map(Optional::of).defaultIfEmpty(Optional.empty())
                .flatMap(optionalJobPost -> {
                    if (optionalJobPost.isPresent()) {
                        jobPost.setId(id);
                        return jobPostRepository.save(jobPost);
                    }

                    return Mono.empty();
                });
    }

    public Mono<Void> deleteById(String id) {
        return jobPostRepository.deleteById(id);
    }


    public Mono<String> publishMessage(String id) {
        return jobPostRepository.findById(id)
                .flatMap(post -> messagePublishService.publishMessage(convertToJson(post)));
    }

    public List<JobPostCompanyDetails> getJobPostCompanyDetails(){
        return jobPostDAOImpl.getJobPostCompanyDetails();
    }

    private String convertToJson(JobPost posts) {
        ObjectMapper mapper = new ObjectMapper();
        String json = " ";
        try {
            json = mapper.writeValueAsString(posts);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        return json;
    }
}
