package com.mongo.reactivie.service;

import java.util.Map;
import org.apache.commons.collections4.map.HashedMap;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.reactive.ReactiveKafkaProducerTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

@Service
public class MessagePublishService {

    private final ReactiveKafkaProducerTemplate<String, String> reactiveKafkaProducerTemplate;

    public MessagePublishService(
            ReactiveKafkaProducerTemplate<String, String> reactiveKafkaProducerTemplate) {
        this.reactiveKafkaProducerTemplate = reactiveKafkaProducerTemplate;
    }

    public Mono<String> publishMessage(String jsonString) {
        ProducerRecord<String, String> record = new ProducerRecord<>(
                "TOPIC", jsonString);
        Map<String, String> headers = new HashedMap<>();
        headers.forEach((key, value) -> record.headers().add(key, value.getBytes()));
        reactiveKafkaProducerTemplate.send(record).subscribe();
        return Mono.just("Message published successfully");
    }

}
